@@ -98,12 +98,18 @@ extern "C" {
 #define SFGAO_HASPROPSHEET	0x00000040L
 #define SFGAO_DROPTARGET	0x00000100L
 #define SFGAO_CAPABILITYMASK	0x00000177L
+#define SFGAO_ISSLOW            0x00004000L
 #define SFGAO_GHOSTED		0x00008000L
 #define SFGAO_LINK		0x00010000L
 #define SFGAO_SHARE		0x00020000L
 #define SFGAO_READONLY		0x00040000L
 #define SFGAO_HIDDEN		0x00080000L
-#define SFGAO_DISPLAYATTRMASK	0x000F0000L
+#define SFGAO_DISPLAYATTRMASK   (SFGAO_ISSLOW \
+				 | SFGAO_GHOSTED \
+				 | SFGAO_LINK \
+				 | SFGAO_SHARE \
+				 | SFGAO_READONLY \
+				 | SFGAO_HIDDEN)
 #define SFGAO_FILESYSANCESTOR	0x10000000L
 #define SFGAO_FOLDER		0x20000000L
 #define SFGAO_FILESYSTEM	0x40000000L
@@ -554,7 +560,6 @@ typedef IEnumIDList *LPENUMIDLIST;
 #define IEnumIDList_Clone(T,a) (T)->lpVtbl->Clone(T,a)
 #endif
 
-#undef INTERFACE
 #define INTERFACE IObjMgr
 DECLARE_INTERFACE_(IObjMgr, IUnknown)
 {
@@ -564,8 +569,8 @@ DECLARE_INTERFACE_(IObjMgr, IUnknown)
 	STDMETHOD(Append)(THIS_ IUnknown*) PURE;
 	STDMETHOD(Remove)(THIS_ IUnknown*) PURE;
 };
-
 #undef INTERFACE
+
 #define INTERFACE IContextMenu
 DECLARE_INTERFACE_(IContextMenu,IUnknown)
 {
@@ -576,6 +581,7 @@ DECLARE_INTERFACE_(IContextMenu,IUnknown)
 	STDMETHOD(InvokeCommand)(THIS_ LPCMINVOKECOMMANDINFO) PURE;
 	STDMETHOD(GetCommandString)(THIS_ UINT,UINT,PUINT,LPSTR,UINT) PURE;
 };
+#undef INTERFACE
 typedef IContextMenu *LPCONTEXTMENU;
 
 #ifdef COBJMACROS
@@ -587,7 +593,6 @@ typedef IContextMenu *LPCONTEXTMENU;
 #define IContextMenu_GetCommandString(T,a,b,c,d,e) (T)->lpVtbl->GetCommandString(T,a,b,c,d,e)
 #endif
 
-#undef INTERFACE
 #define INTERFACE IContextMenu2
 DECLARE_INTERFACE_(IContextMenu2,IContextMenu)
 {
@@ -599,9 +604,9 @@ DECLARE_INTERFACE_(IContextMenu2,IContextMenu)
 	STDMETHOD(GetCommandString)(THIS_ UINT,UINT,PUINT,LPSTR,UINT) PURE;
 	STDMETHOD(HandleMenuMsg)(THIS_ UINT,WPARAM,LPARAM) PURE;
 };
+#undef INTERFACE
 typedef IContextMenu2 *LPCONTEXTMENU2;
 
-#undef INTERFACE
 #define INTERFACE IContextMenu3
 DECLARE_INTERFACE_(IContextMenu3,IContextMenu2)
 {
@@ -614,6 +619,7 @@ DECLARE_INTERFACE_(IContextMenu3,IContextMenu2)
  STDMETHOD(HandleMenuMsg)(THIS_ UINT,WPARAM,LPARAM) PURE;
  STDMETHOD(HandleMenuMsg2)(THIS_ UINT,WPARAM,LPARAM,LRESULT*) PURE;
 };
+#undef INTERFACE
 typedef IContextMenu3 *LPCONTEXTMENU3;
 
 #if (_WIN32_IE >= 0x0500)
@@ -684,7 +690,6 @@ typedef enum {
 #define IContextMenu3_HandleMenuMsg2(T,a,b,c,d) (T)->lpVtbl->HandleMenuMsg(T,a,b,c,d)
 #endif
 
-#undef INTERFACE
 #define INTERFACE IColumnProvider
 DECLARE_INTERFACE_(IColumnProvider,IUnknown)
 {
@@ -695,9 +700,9 @@ DECLARE_INTERFACE_(IColumnProvider,IUnknown)
 	STDMETHOD(GetColumnInfo)(THIS_ DWORD,SHCOLUMNINFO*) PURE;
 	STDMETHOD(GetItemData)(THIS_ LPCSHCOLUMNID,LPCSHCOLUMNDATA,VARIANT*) PURE;
 };
+#undef INTERFACE
 #endif  /* _WIN32_IE >= 0x0500 */
 
-#undef INTERFACE
 #define INTERFACE IQueryInfo
 DECLARE_INTERFACE_(IQueryInfo,IUnknown)
 {
@@ -707,8 +712,8 @@ DECLARE_INTERFACE_(IQueryInfo,IUnknown)
 	STDMETHOD(GetInfoTip)(THIS_ DWORD,WCHAR**) PURE;
 	STDMETHOD(GetInfoFlags)(THIS_ DWORD*) PURE;
 };
-
 #undef INTERFACE
+
 #define INTERFACE IShellExtInit
 DECLARE_INTERFACE_(IShellExtInit, IUnknown)
 {
@@ -717,9 +722,9 @@ DECLARE_INTERFACE_(IShellExtInit, IUnknown)
 	STDMETHOD_(ULONG,Release)(THIS) PURE;
 	STDMETHOD(Initialize)(THIS_ LPCITEMIDLIST,LPDATAOBJECT,HKEY) PURE;
 };
+#undef INTERFACE
 typedef IShellExtInit *LPSHELLEXTINIT;
 
-#undef INTERFACE
 #define INTERFACE IShellPropSheetExt
 DECLARE_INTERFACE_(IShellPropSheetExt, IUnknown)
 {
@@ -729,9 +734,9 @@ DECLARE_INTERFACE_(IShellPropSheetExt, IUnknown)
 	STDMETHOD(AddPages)(THIS_ LPFNADDPROPSHEETPAGE,LPARAM) PURE;
 	STDMETHOD(ReplacePage)(THIS_ UINT,LPFNADDPROPSHEETPAGE,LPARAM) PURE;
 };
+#undef INTERFACE
 typedef IShellPropSheetExt *LPSHELLPROPSHEETEXT;
 
-#undef INTERFACE
 #define INTERFACE IExtractIconA
 DECLARE_INTERFACE_(IExtractIconA, IUnknown)
 {
@@ -741,6 +746,7 @@ DECLARE_INTERFACE_(IExtractIconA, IUnknown)
 	STDMETHOD(GetIconLocation)(THIS_ UINT,LPSTR,UINT,int*,PUINT) PURE;
 	STDMETHOD(Extract)(THIS_ LPCSTR,UINT,HICON*,HICON*,UINT) PURE;
 };
+#undef INTERFACE
 typedef IExtractIconA *LPEXTRACTICONA;
 
 #ifdef COBJMACROS
@@ -751,7 +757,6 @@ typedef IExtractIconA *LPEXTRACTICONA;
 #define IExtractIconA_Extract(T,a,b,c,d,e) (T)->lpVtbl->Extract(T,a,b,c,d,e)
 #endif
 
-#undef INTERFACE
 #define INTERFACE IExtractIconW
 DECLARE_INTERFACE_(IExtractIconW, IUnknown)
 {
@@ -761,6 +766,7 @@ DECLARE_INTERFACE_(IExtractIconW, IUnknown)
 	STDMETHOD(GetIconLocation)(THIS_ UINT,LPWSTR,UINT,int*,PUINT) PURE;
 	STDMETHOD(Extract)(THIS_ LPCWSTR,UINT,HICON*,HICON*,UINT) PURE;
 };
+#undef INTERFACE
 typedef IExtractIconW *LPEXTRACTICONW;
 
 #ifdef COBJMACROS
@@ -779,7 +785,6 @@ typedef IExtractIconW *LPEXTRACTICONW;
 #define LPEXTRACTICON LPEXTRACTICONA
 #endif
 
-#undef INTERFACE
 #define INTERFACE IShellLinkA
 DECLARE_INTERFACE_(IShellLinkA, IUnknown)
 {
@@ -805,6 +810,7 @@ DECLARE_INTERFACE_(IShellLinkA, IUnknown)
 	STDMETHOD(Resolve)(THIS_ HWND,DWORD) PURE;
 	STDMETHOD(SetPath)(THIS_ LPCSTR) PURE;
 };
+#undef INTERFACE
 
 #ifdef COBJMACROS
 #define IShellLinkA_QueryInterface(T,a,b) (T)->lpVtbl->QueryInterface(T,a,b)
@@ -830,7 +836,6 @@ DECLARE_INTERFACE_(IShellLinkA, IUnknown)
 #define IShellLinkA_SetPath(T,a) (T)->lpVtbl->SetPath(T,a)
 #endif
 
-#undef INTERFACE
 #define INTERFACE IShellLinkW
 DECLARE_INTERFACE_(IShellLinkW, IUnknown)
 {
@@ -856,6 +861,7 @@ DECLARE_INTERFACE_(IShellLinkW, IUnknown)
 	STDMETHOD(Resolve)(THIS_ HWND,DWORD) PURE;
 	STDMETHOD(SetPath)(THIS_ LPCWSTR) PURE;
 };
+#undef INTERFACE
 
 #ifdef COBJMACROS
 #define IShellLinkW_QueryInterface(T,a,b) (T)->lpVtbl->QueryInterface(T,a,b)
@@ -881,7 +887,6 @@ DECLARE_INTERFACE_(IShellLinkW, IUnknown)
 #define IShellLinkW_SetPath(T,a) (T)->lpVtbl->SetPath(T,a)
 #endif
 
-#undef INTERFACE
 #define INTERFACE IShellFolder
 DECLARE_INTERFACE_(IShellFolder, IUnknown)
 {
@@ -899,6 +904,7 @@ DECLARE_INTERFACE_(IShellFolder, IUnknown)
 	STDMETHOD(GetDisplayNameOf)(THIS_ LPCITEMIDLIST,DWORD,LPSTRRET) PURE;
 	STDMETHOD(SetNameOf)(THIS_ HWND,LPCITEMIDLIST,LPCOLESTR,DWORD,LPITEMIDLIST*) PURE;
 };
+#undef INTERFACE
 typedef IShellFolder *LPSHELLFOLDER;
 
 #ifdef COBJMACROS
@@ -922,7 +928,6 @@ typedef IShellFolder *LPSHELLFOLDER;
 DECLARE_ENUMERATOR_(IEnumExtraSearch,LPEXTRASEARCH);
 typedef IEnumExtraSearch *LPENUMEXTRASEARCH;
 
-#undef INTERFACE
 #define INTERFACE IShellFolder2
 DECLARE_INTERFACE_(IShellFolder2, IShellFolder)
 {
@@ -947,6 +952,7 @@ DECLARE_INTERFACE_(IShellFolder2, IShellFolder)
 	STDMETHOD(GetDetailsOf)(THIS_ LPCITEMIDLIST,UINT,SHELLDETAILS*) PURE;
 	STDMETHOD(MapColumnToSCID)(THIS_ UINT,SHCOLUMNID*) PURE;
 };
+#undef INTERFACE
 typedef IShellFolder2 *LPSHELLFOLDER2;
 
 #ifdef COBJMACROS
@@ -973,7 +979,6 @@ typedef IShellFolder2 *LPSHELLFOLDER2;
 #endif
 
 #endif /* _WIN32_IE >= 0x0500 */
-#undef INTERFACE
 #define INTERFACE ICopyHook
 DECLARE_INTERFACE_(ICopyHook, IUnknown)
 {
@@ -982,9 +987,9 @@ DECLARE_INTERFACE_(ICopyHook, IUnknown)
 	STDMETHOD_(ULONG,Release)(THIS) PURE;
 	STDMETHOD_(UINT,CopyCallback)(THIS_ HWND,UINT,UINT,LPCSTR,DWORD,LPCSTR,DWORD) PURE;
 };
+#undef INTERFACE
 typedef ICopyHook *LPCOPYHOOK;
 
-#undef INTERFACE
 #define INTERFACE IFileViewerSite
 DECLARE_INTERFACE(IFileViewerSite)
 {
@@ -994,9 +999,9 @@ DECLARE_INTERFACE(IFileViewerSite)
 	STDMETHOD(SetPinnedWindow)(THIS_ HWND) PURE;
 	STDMETHOD(GetPinnedWindow)(THIS_ HWND*) PURE;
 };
+#undef INTERFACE
 typedef IFileViewerSite *LPFILEVIEWERSITE;
 
-#undef INTERFACE
 #define INTERFACE IFileViewer
 DECLARE_INTERFACE(IFileViewer)
 {
@@ -1007,9 +1012,9 @@ DECLARE_INTERFACE(IFileViewer)
 	STDMETHOD(Show)(THIS_ LPFVSHOWINFO) PURE;
 	STDMETHOD(PrintTo)(THIS_ LPSTR,BOOL) PURE;
 };
+#undef INTERFACE
 typedef IFileViewer *LPFILEVIEWER;
 
-#undef INTERFACE
 #define INTERFACE IFileSystemBindData
 DECLARE_INTERFACE_(IFileSystemBindData, IUnknown)
 {
@@ -1019,8 +1024,8 @@ DECLARE_INTERFACE_(IFileSystemBindData, IUnknown)
 	STDMETHOD(SetFindData)(THIS_ const WIN32_FIND_DATAW*) PURE;
 	STDMETHOD(GetFindData)(THIS_ WIN32_FIND_DATAW*) PURE;
 };
-
 #undef INTERFACE
+
 #define INTERFACE IPersistFolder
 DECLARE_INTERFACE_(IPersistFolder,IPersist)
 {
@@ -1030,6 +1035,7 @@ DECLARE_INTERFACE_(IPersistFolder,IPersist)
 	STDMETHOD(GetClassID)(THIS_ CLSID*) PURE;
 	STDMETHOD(Initialize)(THIS_ LPCITEMIDLIST) PURE;
 };
+#undef INTERFACE
 typedef IPersistFolder *LPPERSISTFOLDER;
 
 #ifdef COBJMACROS
@@ -1042,7 +1048,6 @@ typedef IPersistFolder *LPPERSISTFOLDER;
 
 #if (_WIN32_IE >= 0x0400 || _WIN32_WINNT >= 0x0500)
 
-#undef INTERFACE
 #define INTERFACE IPersistFolder2
 DECLARE_INTERFACE_(IPersistFolder2,IPersistFolder)
 {
@@ -1053,6 +1058,7 @@ DECLARE_INTERFACE_(IPersistFolder2,IPersistFolder)
 	STDMETHOD(Initialize)(THIS_ LPCITEMIDLIST) PURE;
 	STDMETHOD(GetCurFolder)(THIS_ LPITEMIDLIST*) PURE;
 };
+#undef INTERFACE
 typedef IPersistFolder2 *LPPERSISTFOLDER2;
 
 #ifdef COBJMACROS
@@ -1068,7 +1074,6 @@ typedef IPersistFolder2 *LPPERSISTFOLDER2;
 
 #if (_WIN32_IE >= 0x0500)
 
-#undef INTERFACE
 #define INTERFACE IPersistFolder3
 DECLARE_INTERFACE_(IPersistFolder3,IPersistFolder2)
 {
@@ -1081,6 +1086,7 @@ DECLARE_INTERFACE_(IPersistFolder3,IPersistFolder2)
 	STDMETHOD(InitializeEx)(THIS_ IBindCtx*,LPCITEMIDLIST,const PERSIST_FOLDER_TARGET_INFO*) PURE;
 	STDMETHOD(GetFolderTargetInfo)(THIS_ PERSIST_FOLDER_TARGET_INFO*) PURE;
 };
+#undef INTERFACE
 typedef IPersistFolder3 *LPPERSISTFOLDER3;
 
 #ifdef COBJMACROS
@@ -1096,10 +1102,9 @@ typedef IPersistFolder3 *LPPERSISTFOLDER3;
 
 #endif /* _WIN32_IE >= 0x0500 */
 
-typedef interface IShellBrowser *LPSHELLBROWSER;
-typedef interface IShellView *LPSHELLVIEW;
+typedef _COM_interface IShellBrowser *LPSHELLBROWSER;
+typedef _COM_interface IShellView *LPSHELLVIEW;
 
-#undef INTERFACE
 #define INTERFACE IShellBrowser
 DECLARE_INTERFACE_(IShellBrowser,IOleWindow)
 {
@@ -1122,6 +1127,7 @@ DECLARE_INTERFACE_(IShellBrowser,IOleWindow)
 	STDMETHOD(OnViewWindowActive)(THIS_ LPSHELLVIEW) PURE;
 	STDMETHOD(SetToolbarItems)(THIS_ LPTBBUTTON,UINT,UINT) PURE;
 };
+#undef INTERFACE
 
 #ifdef COBJMACROS
 #define IShellBrowser_QueryInterface(T,a,b) (T)->lpVtbl->QueryInterface(T,a,b)
@@ -1144,7 +1150,6 @@ DECLARE_INTERFACE_(IShellBrowser,IOleWindow)
 #define IShellBrowser_SetToolbarItems(T,a,b,c) (T)->lpVtbl->SetToolbarItems(T,a,b,c)
 #endif
 
-#undef INTERFACE
 #define INTERFACE IShellView
 DECLARE_INTERFACE_(IShellView,IOleWindow)
 {
@@ -1169,6 +1174,7 @@ DECLARE_INTERFACE_(IShellView,IOleWindow)
 	STDMETHOD(SelectItem)(THIS_ LPCITEMIDLIST,UINT) PURE;
 	STDMETHOD(GetItemObject)(THIS_ UINT,REFIID,PVOID*) PURE;
 };
+#undef INTERFACE
 
 #ifdef COBJMACROS
 #define IShellView_QueryInterface(T,a,b) (T)->lpVtbl->QueryInterface(T,a,b)
@@ -1193,7 +1199,6 @@ DECLARE_INTERFACE_(IShellView,IOleWindow)
 #define IShellView_GetItemObject(T,a,b,c) (T)->lpVtbl->GetItemObject(T,a,b,c)
 #endif
 
-#undef INTERFACE
 #define INTERFACE ICommDlgBrowser
 DECLARE_INTERFACE_(ICommDlgBrowser,IUnknown)
 {
@@ -1204,6 +1209,7 @@ DECLARE_INTERFACE_(ICommDlgBrowser,IUnknown)
 	STDMETHOD(OnStateChange)(THIS_ IShellView*,ULONG) PURE;
 	STDMETHOD(IncludeObject)(THIS_ IShellView*,LPCITEMIDLIST) PURE;
 };
+#undef INTERFACE
 typedef ICommDlgBrowser *LPCOMMDLGBROWSER;
 
 #ifdef COBJMACROS
@@ -1225,7 +1231,6 @@ typedef struct _SV2CVW2_PARAMS {
 	SHELLVIEWID const *pvid;
 	HWND hwndView;
 } SV2CVW2_PARAMS,*LPSV2CVW2_PARAMS;
-#undef INTERFACE
 #define INTERFACE IShellView2
 DECLARE_INTERFACE_(IShellView2,IShellView)
 {
@@ -1253,8 +1258,8 @@ DECLARE_INTERFACE_(IShellView2,IShellView)
 	STDMETHOD(GetView)(THIS_ SHELLVIEWID*,ULONG) PURE;
 	STDMETHOD(CreateViewWindow2)(THIS_ LPSV2CVW2_PARAMS) PURE;
 };
-
 #undef INTERFACE
+
 #define INTERFACE IShellExecuteHookA
 DECLARE_INTERFACE_(IShellExecuteHookA,IUnknown)
 {
@@ -1264,6 +1269,7 @@ DECLARE_INTERFACE_(IShellExecuteHookA,IUnknown)
 	STDMETHOD(Execute)(THIS_ LPSHELLEXECUTEINFOA) PURE;
 };
 #undef INTERFACE
+
 #define INTERFACE IShellExecuteHookW
 DECLARE_INTERFACE_(IShellExecuteHookW,IUnknown)
 {
@@ -1272,8 +1278,8 @@ DECLARE_INTERFACE_(IShellExecuteHookW,IUnknown)
 	STDMETHOD_(ULONG,Release)(THIS) PURE;
 	STDMETHOD(Execute)(THIS_ LPSHELLEXECUTEINFOW) PURE;
 };
-
 #undef INTERFACE
+
 #define INTERFACE IShellIcon
 DECLARE_INTERFACE_(IShellIcon,IUnknown)
 {
@@ -1282,6 +1288,7 @@ DECLARE_INTERFACE_(IShellIcon,IUnknown)
 	STDMETHOD_(ULONG,Release)(THIS) PURE;
 	STDMETHOD(GetIconOf)(THIS_ LPCITEMIDLIST,UINT,PINT) PURE;
 };
+#undef INTERFACE
 typedef IShellIcon *LPSHELLICON;
 
 typedef struct {
@@ -1315,7 +1322,6 @@ typedef struct {
 #define SSF_HIDEICONS 0x4000
 #define SSF_NOCONFIRMRECYCLE 0x8000
 
-#undef  INTERFACE
 #define INTERFACE IShellIconOverlayIdentifier
 DECLARE_INTERFACE_(IShellIconOverlayIdentifier,IUnknown)
 {
@@ -1326,6 +1332,7 @@ DECLARE_INTERFACE_(IShellIconOverlayIdentifier,IUnknown)
 	STDMETHOD(GetOverlayInfo)(THIS_ LPWSTR,int,int*,DWORD*) PURE;
 	STDMETHOD(GetPriority)(THIS_ int*) PURE;
 };
+#undef INTERFACE
 
 #define ISIOI_ICONFILE 0x00000001
 #define ISIOI_ICONINDEX 0x00000002
@@ -1373,7 +1380,6 @@ typedef struct
 } SHDRAGIMAGE, *LPSHDRAGIMAGE;
 #pragma pack(pop)
 
-#undef INTERFACE
 #define INTERFACE IDragSourceHelper
 DECLARE_INTERFACE_(IDragSourceHelper, IUnknown)
 {
@@ -1383,8 +1389,8 @@ DECLARE_INTERFACE_(IDragSourceHelper, IUnknown)
 	STDMETHOD (InitializeFromBitmap)(THIS_ LPSHDRAGIMAGE pshdi, IDataObject* pDataObject) PURE;
 	STDMETHOD (InitializeFromWindow)(THIS_ HWND hwnd, POINT* ppt, IDataObject* pDataObject) PURE;
 };
-
 #undef INTERFACE
+
 #define INTERFACE IDropTargetHelper
 DECLARE_INTERFACE_(IDropTargetHelper, IUnknown)
 {
@@ -1397,8 +1403,17 @@ DECLARE_INTERFACE_(IDropTargetHelper, IUnknown)
 	STDMETHOD (Drop)(THIS_ IDataObject* pDataObject, POINT* ppt, DWORD dwEffect) PURE;
 	STDMETHOD (Show)(THIS_ BOOL fShow) PURE;
 };
+#undef INTERFACE
 #endif /* _WIN32_IE >= 0x0500 */
 
+#if (_WIN32_WINNT >= 0x0500)
+BOOL WINAPI PathResolve(LPWSTR, LPCWSTR*, UINT);
+#define PRF_VERIFYEXISTS            0x0001
+#define PRF_TRYPROGRAMEXTENSIONS    (0x0002 | PRF_VERIFYEXISTS)
+#define PRF_FIRSTDIRDEF             0x0004
+#define PRF_DONTFINDLNK             0x0008
+#endif
+
 void WINAPI SHAddToRecentDocs(UINT,PCVOID);
 LPITEMIDLIST WINAPI SHBrowseForFolderA(PBROWSEINFOA);
 LPITEMIDLIST WINAPI SHBrowseForFolderW(PBROWSEINFOW);
