@@ -1,5 +1,8 @@
 /*
  * stdio.h
+ * This file has no copyright assigned and is placed in the Public Domain.
+ * This file is a part of the mingw-runtime package.
+ * No warranty is given; refer to the file DISCLAIMER within the package.
  *
  * Definitions of types and prototypes of functions for standard input and
  * output.
@@ -7,25 +10,6 @@
  * NOTE: The file manipulation functions provided by Microsoft seem to
  * work with either slash (/) or backslash (\) as the directory separator.
  *
- * This file is part of the Mingw32 package.
- *
- * Contributors:
- *  Created by Colin Peters <colin@bird.fu.is.saga-u.ac.jp>
- *
- *  THIS SOFTWARE IS NOT COPYRIGHTED
- *
- *  This source code is offered for use in the public domain. You may
- *  use, modify or distribute it freely.
- *
- *  This code is distributed in the hope that it will be useful but
- *  WITHOUT ANY WARRANTY. ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
- *  DISCLAIMED. This includes but is not limited to warranties of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- *
- * $Revision: 1.15 $
- * $Author: earnie $
- * $Date: 2003/10/10 15:02:04 $
- *
  */
 
 #ifndef _STDIO_H_
@@ -86,6 +70,9 @@
  * Redefining these macros does not effect the CRT functions.
  */
 #define _P_tmpdir   "\\"
+#ifndef __STRICT_ANSI__
+#define P_tmpdir _P_tmpdir
+#endif
 #define _wP_tmpdir  L"\\"
 
 /*
@@ -114,20 +101,11 @@
 #define	BUFSIZ	512
 
 /* Constants for nOrigin indicating the position relative to which fseek
- * sets the file position. Enclosed in ifdefs because io.h could also
- * define them. (Though not anymore since io.h includes this file now.) */
-#ifndef	SEEK_SET
-#define SEEK_SET	(0)
-#endif
-
-#ifndef	SEEK_CUR
-#define	SEEK_CUR	(1)
-#endif
-
-#ifndef	SEEK_END
-#define SEEK_END	(2)
-#endif
-
+ * sets the file position.  Defined unconditionally since ISO and POSIX
+ * say they are defined here.  */
+#define SEEK_SET 0
+#define SEEK_CUR 1
+#define SEEK_END 2
 
 #ifndef	RC_INVOKED
 
@@ -197,11 +175,18 @@ _CRTIMP int __cdecl	remove (const char*);
 _CRTIMP int __cdecl	rename (const char*, const char*);
 _CRTIMP FILE* __cdecl	tmpfile (void);
 _CRTIMP char* __cdecl	tmpnam (char*);
+
+#ifndef __STRICT_ANSI__
 _CRTIMP char* __cdecl	_tempnam (const char*, const char*);
+_CRTIMP int __cdecl	_rmtmp(void);
+_CRTIMP int __cdecl	_unlink (const char*);
 
 #ifndef	NO_OLDNAMES
 _CRTIMP char* __cdecl	tempnam (const char*, const char*);
+_CRTIMP int __cdecl	rmtmp(void);
+_CRTIMP int __cdecl	unlink (const char*);
 #endif
+#endif /* __STRICT_ANSI__ */
 
 _CRTIMP int __cdecl	setvbuf (FILE*, char*, int, size_t);
 
@@ -265,28 +250,28 @@ _CRTIMP int __cdecl	_flsbuf (int, FILE*);
 __CRT_INLINE int __cdecl getc (FILE* __F)
 {
   return (--__F->_cnt >= 0)
-    ?  (int) *__F->_ptr++
+    ?  (int) (unsigned char) *__F->_ptr++
     : _filbuf (__F);
 }
 
 __CRT_INLINE int __cdecl putc (int __c, FILE* __F)
 {
   return (--__F->_cnt >= 0)
-    ?  (int)(*__F->_ptr++ = (char)__c)
+    ?  (int) (unsigned char) (*__F->_ptr++ = (char)__c)
     :  _flsbuf (__c, __F);
 }
 
 __CRT_INLINE int __cdecl getchar (void)
 {
   return (--stdin->_cnt >= 0)
-    ?  (int) *stdin->_ptr++
+    ?  (int) (unsigned char) *stdin->_ptr++
     : _filbuf (stdin);
 }
 
 __CRT_INLINE int __cdecl putchar(int __c)
 {
   return (--stdout->_cnt >= 0)
-    ?  (int)(*stdout->_ptr++ = (char)__c)
+    ?  (int) (unsigned char) (*stdout->_ptr++ = (char)__c)
     :  _flsbuf (__c, stdout);}
 
 #else  /* Use library functions.  */
@@ -313,6 +298,16 @@ _CRTIMP int __cdecl	fseek (FILE*, long, int);
 _CRTIMP long __cdecl	ftell (FILE*);
 _CRTIMP void __cdecl	rewind (FILE*);
 
+#if __MSVCRT_VERSION__ >= 0x800
+_CRTIMP int __cdecl	_fseek_nolock (FILE*, long, int);
+_CRTIMP long __cdecl	_ftell_nolock (FILE*);
+
+_CRTIMP int __cdecl	_fseeki64 (FILE*, __int64, int);
+_CRTIMP __int64 __cdecl	_ftelli64 (FILE*);
+_CRTIMP int __cdecl	_fseeki64_nolock (FILE*, __int64, int);
+_CRTIMP __int64 __cdecl	_ftelli64_nolock (FILE*);
+#endif
+
 #ifdef __USE_MINGW_FSEEK  /* These are in libmingwex.a */
 /*
  * Workaround for limitations on win9x where a file contents are
@@ -320,7 +315,7 @@ _CRTIMP void __cdecl	rewind (FILE*);
  */
 
 int __cdecl __mingw_fseek (FILE *, long, int);
-int __cdecl __mingw_fwrite (const void*, size_t, size_t, FILE*);
+size_t __cdecl __mingw_fwrite (const void*, size_t, size_t, FILE*);
 #define fseek(fp, offset, whence)  __mingw_fseek(fp, offset, whence)
 #define fwrite(buffer, size, count, fp)  __mingw_fwrite(buffer, size, count, fp)
 #endif /* __USE_MINGW_FSEEK */
@@ -347,9 +342,20 @@ _CRTIMP int __cdecl	fsetpos (FILE*, const fpos_t*);
  * Error Functions
  */
 
-_CRTIMP void __cdecl	clearerr (FILE*);
 _CRTIMP int __cdecl	feof (FILE*);
 _CRTIMP int __cdecl	ferror (FILE*);
+
+#ifdef __cplusplus
+inline int __cdecl feof (FILE* __F)
+  { return __F->_flag & _IOEOF; }
+inline int __cdecl ferror (FILE* __F)
+  { return __F->_flag & _IOERR; }
+#else
+#define feof(__F)     ((__F)->_flag & _IOEOF)
+#define ferror(__F)   ((__F)->_flag & _IOERR)
+#endif
+
+_CRTIMP void __cdecl	clearerr (FILE*);
 _CRTIMP void __cdecl	perror (const char*);
 
 
@@ -374,23 +380,53 @@ _CRTIMP int __cdecl	_fputchar (int);
 _CRTIMP FILE* __cdecl	_fdopen (int, const char*);
 _CRTIMP int __cdecl	_fileno (FILE*);
 _CRTIMP int __cdecl	_fcloseall(void);
+_CRTIMP FILE* __cdecl	_fsopen(const char*, const char*, int);
 #ifdef __MSVCRT__
 _CRTIMP int __cdecl	_getmaxstdio(void);
 _CRTIMP int __cdecl	_setmaxstdio(int);
 #endif
 
-#define _fileno(__F) ((__F)->_file)
+#if __MSVCRT_VERSION__ >= 0x800
+_CRTIMP int __cdecl _set_printf_count_output(int);
+_CRTIMP int __cdecl _get_printf_count_output(void);
+#endif
 
 #ifndef _NO_OLDNAMES
 _CRTIMP int __cdecl	fgetchar (void);
 _CRTIMP int __cdecl	fputchar (int);
 _CRTIMP FILE* __cdecl	fdopen (int, const char*);
 _CRTIMP int __cdecl	fileno (FILE*);
-
-#define fileno(__F) ((__F)->_file)
-
 #endif	/* Not _NO_OLDNAMES */
 
+#define _fileno(__F) ((__F)->_file)
+#ifndef _NO_OLDNAMES
+#define fileno(__F) ((__F)->_file)
+#endif
+
+#if defined (__MSVCRT__) && !defined (__NO_MINGW_LFS)
+#include <sys/types.h>
+__CRT_INLINE FILE* __cdecl fopen64 (const char* filename, const char* mode)
+{
+  return fopen (filename, mode); 
+}
+
+int __cdecl fseeko64 (FILE*, off64_t, int);
+
+#ifdef __USE_MINGW_FSEEK
+int __cdecl __mingw_fseeko64 (FILE *, off64_t, int);
+#define fseeko64(fp, offset, whence)  __mingw_fseeko64(fp, offset, whence)
+#endif
+
+__CRT_INLINE off64_t __cdecl ftello64 (FILE * stream)
+{
+  fpos_t pos;
+  if (fgetpos(stream, &pos))
+    return  -1LL;
+  else
+   return ((off64_t) pos);
+}
+#endif /* __NO_MINGW_LFS */
+
 #endif	/* Not __STRICT_ANSI__ */
 
 /* Wide  versions */
