@@ -267,21 +267,23 @@ extern "C" {
 #define SP_OUTOFMEMORY	(-5)
 #define SP_USERABORT	(-3)
 #define SP_APPABORT	(-2)
-#define BLACKNESS	0x42
-#define NOTSRCERASE	0x1100A6
-#define NOTSRCCOPY	0x330008
-#define SRCERASE	0x440328
-#define DSTINVERT	0x550009
-#define PATINVERT	0x5A0049
-#define SRCINVERT	0x660046
-#define SRCAND	0x8800C6
-#define MERGEPAINT	0xBB0226
-#define MERGECOPY	0xC000CA
-#define SRCCOPY 0xCC0020
-#define SRCPAINT	0xEE0086
-#define PATCOPY	0xF00021
-#define PATPAINT	0xFB0A09
-#define WHITENESS	0xFF0062
+#define BLACKNESS	0x00000042
+#define NOTSRCERASE	0x001100A6
+#define NOTSRCCOPY	0x00330008
+#define SRCERASE	0x00440328
+#define DSTINVERT	0x00550009
+#define PATINVERT	0x005A0049
+#define SRCINVERT	0x00660046
+#define SRCAND	0x008800C6
+#define MERGEPAINT	0x00BB0226
+#define MERGECOPY	0x00C000CA
+#define SRCCOPY 0x00CC0020
+#define SRCPAINT	0x00EE0086
+#define PATCOPY	0x00F00021
+#define PATPAINT	0x00FB0A09
+#define WHITENESS	0x00FF0062
+#define CAPTUREBLT	0x40000000
+#define NOMIRRORBITMAP	0x80000000
 #define R2_BLACK	1
 #define R2_COPYPEN	13
 #define R2_MASKNOTPEN	3
@@ -557,6 +559,21 @@ extern "C" {
 #define DC_DATATYPE_PRODUCED	21
 #define DC_MANUFACTURER	23
 #define DC_MODEL	24
+#if (WINVER >= 0x0500)
+#define DC_PERSONALITY	25
+#define DC_PRINTRATE	26
+#define DC_PRINTRATEUNIT	27
+#define DC_PRINTERMEM	28
+#define DC_MEDIAREADY	29
+#define DC_STAPLE	30
+#define DC_PRINTRATEPPM	31
+#define DC_COLORDEVICE	32
+#define DC_NUP	33
+#endif
+#if (WINVER >= 0x0501)
+#define DC_MEDIATYPENAMES	34
+#define DC_MEDIATYPES	35
+#endif
 #define DCBA_FACEUPNONE	0
 #define DCBA_FACEUPCENTER	1
 #define DCBA_FACEUPLEFT	2
@@ -567,10 +584,18 @@ extern "C" {
 #define DCBA_FACEDOWNRIGHT	259
 #define FLOODFILLBORDER 0
 #define FLOODFILLSURFACE 1
-#define ETO_CLIPPED 4
-#define ETO_GLYPH_INDEX 16
-#define ETO_OPAQUE 2
-#define ETO_RTLREADING 128
+#define ETO_CLIPPED 0x0004
+#define ETO_GLYPH_INDEX 0x0010
+#define ETO_OPAQUE 0x0002
+#define ETO_NUMERICSLATIN 0x0800
+#define ETO_NUMERICSLOCAL 0x0400
+#define ETO_RTLREADING 0x0080
+#if (_WIN32_WINNT >= 0x0400)
+#define ETO_IGNORELANGUAGE 0x1000
+#endif
+#if (_WIN32_WINNT >= 0x0500)
+#define ETO_PDY 0x2000
+#endif
 #define GDICOMMENT_WINDOWS_METAFILE (-2147483647)
 #define GDICOMMENT_BEGINGROUP 2
 #define GDICOMMENT_ENDGROUP 3
@@ -774,7 +799,9 @@ extern "C" {
 #define GGO_GRAY4_BITMAP 5
 #define GGO_GRAY8_BITMAP 6
 #define GGO_GLYPH_INDEX 128
+#if (WINVER >= 0x0500)
 #define GGO_UNHINTED 256
+#endif
 #define GM_COMPATIBLE 1
 #define GM_ADVANCED 2
 #define MM_ANISOTROPIC 8
@@ -1035,7 +1062,61 @@ extern "C" {
 #define DMPAPER_A2	66
 #define DMPAPER_A3_TRANSVERSE	67
 #define DMPAPER_A3_EXTRA_TRANSVERSE	68
+#if (WINVER >= 0x0500)
+#define DMPAPER_DBL_JAPANESE_POSTCARD	69
+#define DMPAPER_A6	70
+#define DMPAPER_JENV_KAKU2	71
+#define DMPAPER_JENV_KAKU3	72
+#define DMPAPER_JENV_CHOU3	73
+#define DMPAPER_JENV_CHOU4	74
+#define DMPAPER_LETTER_ROTATED	75
+#define DMPAPER_A3_ROTATED	76
+#define DMPAPER_A4_ROTATED	77
+#define DMPAPER_A5_ROTATED	78
+#define DMPAPER_B4_JIS_ROTATED	79
+#define DMPAPER_B5_JIS_ROTATED	80
+#define DMPAPER_JAPANESE_POSTCARD_ROTATED	81
+#define DMPAPER_DBL_JAPANESE_POSTCARD_ROTATED	82
+#define DMPAPER_A6_ROTATED	83
+#define DMPAPER_JENV_KAKU2_ROTATED	84
+#define DMPAPER_JENV_KAKU3_ROTATED	85
+#define DMPAPER_JENV_CHOU3_ROTATED	86
+#define DMPAPER_JENV_CHOU4_ROTATED	87
+#define DMPAPER_B6_JIS	88
+#define DMPAPER_B6_JIS_ROTATED	89
+#define DMPAPER_12X11	90
+#define DMPAPER_JENV_YOU4	91
+#define DMPAPER_JENV_YOU4_ROTATED	92
+#define DMPAPER_P16K	93
+#define DMPAPER_P32K	94
+#define DMPAPER_P32KBIG	95
+#define DMPAPER_PENV_1	96
+#define DMPAPER_PENV_2	97
+#define DMPAPER_PENV_3	98
+#define DMPAPER_PENV_4	99
+#define DMPAPER_PENV_5	100
+#define DMPAPER_PENV_6	101
+#define DMPAPER_PENV_7	102
+#define DMPAPER_PENV_8	103
+#define DMPAPER_PENV_9	104
+#define DMPAPER_PENV_10	105
+#define DMPAPER_P16K_ROTATED	106
+#define DMPAPER_P32K_ROTATED	107
+#define DMPAPER_P32KBIG_ROTATED	108
+#define DMPAPER_PENV_1_ROTATED	109
+#define DMPAPER_PENV_2_ROTATED	110
+#define DMPAPER_PENV_3_ROTATED	111
+#define DMPAPER_PENV_4_ROTATED	112
+#define DMPAPER_PENV_5_ROTATED	113
+#define DMPAPER_PENV_6_ROTATED	114
+#define DMPAPER_PENV_7_ROTATED	115
+#define DMPAPER_PENV_8_ROTATED	116
+#define DMPAPER_PENV_9_ROTATED	117
+#define DMPAPER_PENV_10_ROTATED	118
+#define DMPAPER_LAST	118
+#else
 #define DMPAPER_LAST	68
+#endif
 #define DMPAPER_USER	256
 #define DMBIN_FIRST	1
 #define DMBIN_UPPER	1
@@ -1080,30 +1161,46 @@ extern "C" {
 #define DM_IN_PROMPT	DM_PROMPT
 #define DM_OUT_BUFFER	DM_COPY
 #define DM_OUT_DEFAULT	DM_UPDATE
-#define DM_ORIENTATION 1
-#define DM_PAPERSIZE 2
-#define DM_PAPERLENGTH 4
-#define DM_PAPERWIDTH 8
-#define DM_SCALE 16
-#define DM_COPIES 256
-#define DM_DEFAULTSOURCE 512
-#define DM_PRINTQUALITY 1024
-#define DM_COLOR 2048
-#define DM_DUPLEX 4096
-#define DM_YRESOLUTION 8192
-#define DM_TTOPTION 16384
-#define DM_COLLATE 32768
-#define DM_FORMNAME 65536
-#define DM_LOGPIXELS 0x20000
-#define DM_BITSPERPEL 0x40000
-#define DM_PELSWIDTH 0x80000
-#define DM_PELSHEIGHT 0x100000
-#define DM_DISPLAYFLAGS 0x200000
-#define DM_DISPLAYFREQUENCY 0x400000
-#define DM_ICMMETHOD 0x800000
-#define DM_ICMINTENT 0x1000000
-#define DM_MEDIATYPE 0x2000000
-#define DM_DITHERTYPE 0x4000000
+#define DM_ORIENTATION 0x00000001
+#define DM_PAPERSIZE 0x00000002
+#define DM_PAPERLENGTH 0x00000004
+#define DM_PAPERWIDTH 0x00000008
+#define DM_SCALE 0x00000010
+#define DM_POSITION 0x00000020
+#define DM_COPIES 0x00000100
+#define DM_DEFAULTSOURCE 0x00000200
+#define DM_PRINTQUALITY 0x00000400
+#define DM_COLOR 0x00000800
+#define DM_DUPLEX 0x00001000
+#define DM_YRESOLUTION 0x00002000
+#define DM_TTOPTION 0x00004000
+#define DM_COLLATE 0x00008000
+#define DM_FORMNAME 0x00010000
+#define DM_LOGPIXELS 0x00020000
+#define DM_BITSPERPEL 0x00040000
+#define DM_PELSWIDTH 0x00080000
+#define DM_PELSHEIGHT 0x00100000
+#define DM_DISPLAYFLAGS 0x00200000
+#define DM_DISPLAYFREQUENCY 0x00400000
+#define DM_ICMMETHOD 0x00800000
+#define DM_ICMINTENT 0x01000000
+#define DM_MEDIATYPE 0x02000000
+#define DM_DITHERTYPE 0x04000000
+#if(WINVER >= 0x0500)
+#define DM_PANNINGWIDTH 0x08000000
+#define DM_PANNINGHEIGHT 0x10000000
+#endif
+#if(WINVER >= 0x0501)
+#define DM_DISPLAYFIXEDOUTPUT 0x20000000
+#define DM_DISPLAYORIENTATION 0x00000080
+#define DMDO_DEFAULT 0x00000000
+#define DMDO_90 0x00000001
+#define DMDO_180 0x00000002
+#define DMDO_270 0x00000003
+#define DMDFO_DEFAULT 0x00000000
+#define DMDFO_STRETCH 0x00000001
+#define DMDFO_CENTER 0x00000002
+#endif
 #define DMICMMETHOD_NONE	1
 #define DMICMMETHOD_SYSTEM	2
 #define DMICMMETHOD_DRIVER	3
@@ -1196,12 +1293,12 @@ extern "C" {
 #define AC_DST_NO_ALPHA		0x20
 #define LAYOUT_RTL 1
 #define LAYOUT_BITMAPORIENTATIONPRESERVED 8
-#if (WINVER > 0x400)
+#if (WINVER >= 0x0400)
 #define CS_ENABLE           0x00000001
 #define CS_DISABLE          0x00000002
 #define CS_DELETE_TRANSFORM 0x00000003
 #endif
-#if (WINVER > 0x500)
+#if (WINVER >= 0x0500)
 #define GRADIENT_FILL_RECT_H 0x00
 #define GRADIENT_FILL_RECT_V 0x01
 #define GRADIENT_FILL_TRIANGLE 0x02
@@ -1327,6 +1424,32 @@ typedef struct {
 	DWORD	bV4GammaGreen;
 	DWORD	bV4GammaBlue;
 } BITMAPV4HEADER,*LPBITMAPV4HEADER,*PBITMAPV4HEADER;
+typedef struct { 
+	DWORD	bV5Size; 
+	LONG	bV5Width; 
+	LONG	bV5Height; 
+	WORD	bV5Planes; 
+	WORD	bV5BitCount; 
+	DWORD	bV5Compression; 
+	DWORD	bV5SizeImage; 
+	LONG	bV5XPelsPerMeter; 
+	LONG	bV5YPelsPerMeter; 
+	DWORD	bV5ClrUsed; 
+	DWORD	bV5ClrImportant; 
+	DWORD	bV5RedMask; 
+	DWORD	bV5GreenMask; 
+	DWORD	bV5BlueMask; 
+	DWORD	bV5AlphaMask; 
+	DWORD	bV5CSType; 
+	CIEXYZTRIPLE bV5Endpoints; 
+	DWORD	bV5GammaRed; 
+	DWORD	bV5GammaGreen; 
+	DWORD	bV5GammaBlue; 
+	DWORD	bV5Intent; 
+	DWORD	bV5ProfileData; 
+	DWORD	bV5ProfileSize; 
+	DWORD	bV5Reserved; 
+} BITMAPV5HEADER,*LPBITMAPV5HEADER,*PBITMAPV5HEADER; 
 typedef struct tagFONTSIGNATURE {
 	DWORD	fsUsb[4];
 	DWORD	fsCsb[2];
@@ -2571,7 +2694,10 @@ WINGDIAPI int WINAPI ChoosePixelFormat(HDC,CONST PIXELFORMATDESCRIPTOR*);
 WINGDIAPI HENHMETAFILE WINAPI CloseEnhMetaFile(HDC);
 WINGDIAPI BOOL WINAPI CloseFigure(HDC);
 WINGDIAPI HMETAFILE WINAPI CloseMetaFile(HDC);
+#if (_WIN32_WINDOWS >= 0x0410 || _WIN32_WINNT >= 0x0500)
+WINGDIAPI BOOL WINAPI ColorCorrectPalette(HDC,HPALETTE,DWORD,DWORD);
 WINGDIAPI BOOL WINAPI ColorMatchToTarget(HDC,HDC,DWORD);
+#endif
 WINGDIAPI int WINAPI CombineRgn(HRGN,HRGN,HRGN,int);
 WINGDIAPI BOOL WINAPI CombineTransform(LPXFORM,const XFORM*,const XFORM*);
 WINGDIAPI HENHMETAFILE WINAPI CopyEnhMetaFileA(HENHMETAFILE,LPCSTR);
@@ -2691,6 +2817,10 @@ WINGDIAPI HANDLE WINAPI GetColorSpace(HDC);
 WINGDIAPI HGDIOBJ WINAPI GetCurrentObject(HDC,UINT);
 WINGDIAPI BOOL WINAPI GetCurrentPositionEx(HDC,LPPOINT);
 WINGDIAPI HCURSOR WINAPI GetCursor(void);
+#if (WINVER >= 0x0500)
+WINGDIAPI COLORREF WINAPI GetDCBrushColor(HDC);
+WINGDIAPI COLORREF WINAPI GetDCPenColor(HDC);
+#endif
 WINGDIAPI BOOL WINAPI GetDCOrgEx(HDC,LPPOINT);
 WINGDIAPI int WINAPI GetDeviceCaps(HDC,int);
 WINGDIAPI BOOL WINAPI GetDeviceGammaRamp(HDC,PVOID);
@@ -2751,8 +2881,13 @@ WINGDIAPI BOOL WINAPI GetTextExtentExPointA(HDC,LPCSTR,int,int,LPINT,LPINT,LPSIZ
 WINGDIAPI BOOL WINAPI GetTextExtentExPointW( HDC,LPCWSTR,int,int,LPINT,LPINT,LPSIZE );
 WINGDIAPI BOOL WINAPI GetTextExtentPointA(HDC,LPCSTR,int,LPSIZE);
 WINGDIAPI BOOL WINAPI GetTextExtentPointW(HDC,LPCWSTR,int,LPSIZE);
+#ifdef _WIN32_WCE
+extern BOOL GetTextExtentPoint32A(HDC,LPCSTR,int,LPSIZE);
+extern BOOL GetTextExtentPoint32W( HDC,LPCWSTR,int,LPSIZE);
+#else
 WINGDIAPI BOOL WINAPI GetTextExtentPoint32A(HDC,LPCSTR,int,LPSIZE);
 WINGDIAPI BOOL WINAPI GetTextExtentPoint32W( HDC,LPCWSTR,int,LPSIZE);
+#endif
 WINGDIAPI int WINAPI GetTextFaceA(HDC,int,LPSTR);
 WINGDIAPI int WINAPI GetTextFaceW(HDC,int,LPWSTR);
 WINGDIAPI BOOL WINAPI GetTextMetricsA(HDC,LPTEXTMETRICA);
@@ -2900,7 +3035,7 @@ WINGDIAPI BOOL WINAPI wglUseFontBitmapsA(HDC,DWORD,DWORD,DWORD);
 WINGDIAPI BOOL WINAPI wglUseFontBitmapsW(HDC,DWORD,DWORD,DWORD);
 WINGDIAPI BOOL WINAPI wglUseFontOutlinesA(HDC,DWORD,DWORD,DWORD,FLOAT,FLOAT,int,LPGLYPHMETRICSFLOAT);
 WINGDIAPI BOOL WINAPI wglUseFontOutlinesW(HDC,DWORD,DWORD,DWORD,FLOAT,FLOAT,int,LPGLYPHMETRICSFLOAT);
-#if (WINVER >= 0x0500)
+#if (WINVER >= 0x0410)
 WINGDIAPI BOOL WINAPI AlphaBlend(HDC,int,int,int,int,HDC,int,int,int,int,BLENDFUNCTION);
 WINGDIAPI BOOL WINAPI GradientFill(HDC,PTRIVERTEX,ULONG,PVOID,ULONG,ULONG);
 WINGDIAPI BOOL WINAPI TransparentBlt(HDC,int,int,int,int,HDC,int,int,int,int,UINT);
@@ -2910,6 +3045,10 @@ WINGDIAPI DWORD WINAPI GetFontUnicodeRanges(HDC,LPGLYPHSET);
 WINGDIAPI DWORD WINAPI GetGlyphIndicesA(HDC,LPCSTR,int,LPWORD,DWORD);
 WINGDIAPI DWORD WINAPI GetGlyphIndicesW(HDC,LPCWSTR,int,LPWORD,DWORD);
 #endif
+#if (WINVER >= 0x0500)
+WINGDIAPI DWORD WINAPI GetLayout(HDC);
+WINGDIAPI DWORD WINAPI SetLayout(HDC, DWORD);
+#endif
 
 #ifdef UNICODE
 typedef WCHAR BCHAR;
@@ -2940,6 +3079,7 @@ typedef DISPLAY_DEVICEW DISPLAY_DEVICE, *PDISPLAY_DEVICE, *LPDISPLAY_DEVICE;
 #endif
 #define CopyEnhMetaFile CopyEnhMetaFileW
 #define CopyMetaFile CopyMetaFileW
+#define CreateColorSpace CreateColorSpaceW
 #define CreateDC CreateDCW
 #define CreateEnhMetaFile CreateEnhMetaFileW
 #define CreateFont CreateFontW
@@ -3017,6 +3157,7 @@ typedef DISPLAY_DEVICEA DISPLAY_DEVICE, *PDISPLAY_DEVICE, *LPDISPLAY_DEVICE;
 #endif
 #define CopyEnhMetaFile CopyEnhMetaFileA
 #define CopyMetaFile CopyMetaFileA
+#define CreateColorSpace CreateColorSpaceA
 #define CreateDC CreateDCA
 #define CreateEnhMetaFile CreateEnhMetaFileA
 #define CreateFont CreateFontA
